@@ -4,19 +4,18 @@
  *  
  *  Copyright (C) 2011 Peng Wu <alexepico@gmail.com>
  *  
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef FACADE_CHEWING_TABLE_H
@@ -92,41 +91,79 @@ public:
     /**
      * FacadeChewingTable::load:
      * @options: the pinyin options.
-     * @system: the memory chunk of the system chewing table.
-     * @user: the memory chunk of the user chewing table.
+     * @system_filename: the file name of the system chewing table.
+     * @user_filename: the file name of the user chewing table.
      * @returns: whether the load operation is successful.
      *
-     * Load the system or user chewing table from the memory chunks.
+     * Load the system or user chewing table from the files.
      *
      */
-    bool load(pinyin_option_t options, MemoryChunk * system,
-              MemoryChunk * user){
+    bool load(pinyin_option_t options, const char * system_filename,
+              const char * user_filename){
         reset();
 
+        MemoryChunk * chunk = NULL;
+
         bool result = false;
-        if (system) {
+
+        /* load system chewing table. */
+        if (system_filename) {
+            chunk = new MemoryChunk;
+
+#ifdef LIBPINYIN_USE_MMAP
+            if (!chunk->mmap(system_filename)) {
+                fprintf(stderr, "mmap %s failed!\n", system_filename);
+                return false;
+            }
+#else
+            if (!chunk->load(system_filename)) {
+                fprintf(stderr, "open %s failed!\n", system_filename);
+                return false;
+            }
+#endif
+
             m_system_chewing_table = new ChewingLargeTable(options);
-            result = m_system_chewing_table->load(system) || result;
+            result = m_system_chewing_table->load(chunk) || result;
+            chunk = NULL;
         }
-        if (user) {
+
+        /* load user chewing table */
+        if (user_filename) {
+            chunk = new MemoryChunk;
+
+            if (!chunk->load(user_filename)) {
+                /* hack here: use local Chewing Table to create empty memory chunk. */
+                ChewingLargeTable table(options);
+                table.store(chunk);
+            }
+
             m_user_chewing_table = new ChewingLargeTable(options);
-            result = m_user_chewing_table->load(user) || result;
+            result = m_user_chewing_table->load(chunk) || result;
         }
+
         return result;
     }
 
     /**
      * FacadeChewingTable::store:
-     * @new_user: the memory chunk to store the user chewing table.
+     * @new_user_filename: the file name to store the user chewing table.
      * @returns: whether the store operation is successful.
      *
-     * Store the user chewing table to the memory chunk.
+     * Store the user chewing table to the file.
      *
      */
-    bool store(MemoryChunk * new_user) {
+    bool store(const char * new_user_filename) {
         if (NULL == m_user_chewing_table)
             return false;
-        return m_user_chewing_table->store(new_user);
+
+        /* save user chewing table */
+        MemoryChunk * chunk = new MemoryChunk;
+        bool retval = m_user_chewing_table->store(chunk);
+        assert(retval);
+
+        retval = chunk->save(new_user_filename);
+        delete chunk;
+        return retval;
     }
 
     /**
